@@ -52,7 +52,7 @@
 #' @importFrom Rcpp evalCpp
 #' @importFrom RcppParallel RcppParallelLibs
 #' @useDynLib dodgr, .registration = TRUE
-NULL
+"_PACKAGE"
 
 #' Weighting profiles used to route different modes of transport.
 #'
@@ -111,8 +111,9 @@ NULL
 #' To download the data from that page click on the tick box next to
 #' 'OS Open Roads', scroll to the bottom, click 'Continue' and complete
 #' the form on the subsequent page.
-#' This dataset is open access and can be used under the
-#' \href{https://www.ordnancesurvey.co.uk/business-and-government/licensing/using-creating-data-with-os-products/os-opendata.html}{Open Government License}
+#' This dataset is open access and can be used under
+#' \href{https://www.ordnancesurvey.co.uk/licensing}{these licensing
+#' conditions},
 #' and must be cited as follows:
 #' Contains OS data © Crown copyright and database right (2017)
 #'
